<?php
// Enqueue the necessary scripts and styles
function json_file_editor_enqueue_scripts() {
    if (is_admin()) { // Load only in the admin area
        wp_enqueue_script('ace-editor', 'https://cdnjs.cloudflare.com/ajax/libs/ace/1.4.12/ace.js', array(), null, true);
        wp_enqueue_script('ace-mode-json', 'https://cdnjs.cloudflare.com/ajax/libs/ace/1.4.12/mode-json.js', array('ace-editor'), null, true);
        wp_enqueue_style( 'json-file-editor-styles', plugin_dir_url( __FILE__ ) . 'json-file-editor.css' );
    }
}
add_action('admin_enqueue_scripts', 'json_file_editor_enqueue_scripts');


// Add the settings page to the WordPress admin menu
function json_file_editor_settings_page() {
    add_options_page('JSON File Editor', 'JSON File Editor', 'manage_options', 'json-file-editor-settings', 'json_file_editor_settings_page_content');
}
add_action('admin_menu', 'json_file_editor_settings_page');

// Display the content of the settings page
function json_file_editor_settings_page_content() {
    // Define the directory containing the files (adjust this path as needed)
    $directory_path = ABSPATH . 'wp-content/themes/everyware-theme-lavender/nav-site-builder-json/';
    $last_edited_file = isset($_POST['file_to_edit']) ? sanitize_text_field($_POST['file_to_edit']) : '';

    // Check for form submission
    if ($_SERVER['REQUEST_METHOD'] === 'POST' && current_user_can('manage_options') && check_admin_referer('json_file_editor_save_changes')) {
        $file_to_edit = sanitize_text_field($_POST['file_to_edit']);
        $new_contents = isset($_POST['file_content']) ? $_POST['file_content'] : '';
        $new_contents = stripslashes($new_contents);

        // Test if the JSON is valid
        json_decode($new_contents);
        if (json_last_error() === JSON_ERROR_NONE) {
            // Security check: Ensure the file is within the directory
            $real_path = realpath($directory_path . $file_to_edit);
            if (!$real_path || dirname($real_path) !== realpath($directory_path)) {
                echo '<div class="error"><p>Invalid file path.</p></div>';
                return;
            }
            // Directly write the contents if JSON is valid
            file_put_contents($real_path, $new_contents);
            echo '<div class="updated"><p>File updated successfully.</p></div>';
        } else {
            // Display JSON error
            echo '<div class="error"><p>Invalid JSON format: ' . json_last_error_msg() . '. Please correct and try again.</p></div>';
        }
    }

    // List files in the directory
    $files = scandir($directory_path);
    $files = array_filter($files, function ($file) use ($directory_path) {
        return is_file($directory_path . $file);
    });
    ?>
<div class="wrap">
    <h1>File Editor Settings</h1>
    <div style="display: flex;">
        <!-- File List -->
        <div style="flex: 1; padding-right: 20px; border-right: 1px solid #ddd; max-height: 600px; overflow-y: auto;">
            <h2>All JSON Files</h2>
            <ul id="file-list" style="list-style-type: none; padding: 0;">
                <?php foreach ($files as $file) : ?>
                <li style="margin-bottom: 10px;" class="file_list">
                    <a href="#" class="file-link" data-file="<?php echo esc_attr($file); ?>">
                        <?php echo esc_html($file); ?>
                    </a>
                </li>
                <?php endforeach; ?>
            </ul>
        </div>

        <!-- Textarea for File Content -->
        <div style="flex: 3; margin-left: 20px;">
            <div class="editor-header" style="display: flex; justify-content: space-between; align-items: center;">
                <h2>Edit JSON File: <span id="head-file-name"></span></h2>
                <form method="post" id="file-editor-form" style="margin: 0;">
                    <?php wp_nonce_field('json_file_editor_save_changes'); ?>
                    <input type="hidden" name="file_to_edit" id="file-to-edit"
                        value="<?php echo esc_attr($last_edited_file); ?>">
                    <textarea name="file_content" id="file-content" style="display: none;"></textarea>
                    <input type="submit" value="Save Changes" class="button button-primary">
                </form>
            </div>
            <!-- Ace Editor container -->
            <div id="editor" style="flex: 1; height: calc(100vh - 160px);"></div>
        </div>
    </div>
</div>

<script type="text/javascript">
document.addEventListener('DOMContentLoaded', function() {
    var fileLinks = document.querySelectorAll('.file-link');
    var fileContent = document.getElementById('file-content');
    var fileToEdit = document.getElementById('file-to-edit');
    var headFileName = document.getElementById('head-file-name');
    var form = document.getElementById('file-editor-form');

    // Initialize Ace Editor
    var editor = ace.edit("editor");
    editor.setTheme("ace/theme/monokai");
    editor.session.setMode("ace/mode/json");
    editor.setValue("Please select the file to edit.");
    document.getElementById('editor').style.fontSize='16px';

    fileLinks.forEach(function(link) {
        link.addEventListener('click', function(e) {
            e.preventDefault();

            // Remove active class from all file links
            fileLinks.forEach(function(l) {
                l.classList.remove('active-file');
            });

            // Add active class to the clicked file link
            this.classList.add('active-file');

            var fileName = this.getAttribute('data-file');
            fileToEdit.value = fileName;

            headFileName.textContent = fileName;

            // AJAX call to server to get file content
            var xhr = new XMLHttpRequest();
            xhr.open('POST', '<?php echo admin_url('admin-ajax.php'); ?>', true);
            xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
            xhr.onload = function() {
                if (xhr.status >= 200 && xhr.status < 400) {
                    editor.setValue(xhr.responseText, -1); // -1 moves cursor to the start
                } else {
                    alert("Error loading file content");
                }
            };
            xhr.onerror = function() {
                // There was a connection error of some sort
                alert("Error loading file content");
            };
            xhr.send('action=get_file_content&file_name=' + fileName);
        });
    });

    form.addEventListener('submit', function() {
        if (!fileToEdit.value) {
            alert('Please select a file to edit.');
            event.preventDefault();
        }
        document.getElementById('file-content').value = editor.getSession().getValue();
    });

    if (fileToEdit.value) {
        // Trigger click on the corresponding file link
        document.querySelector(`.file-link[data-file="${fileToEdit.value}"]`).click();
    }
});
</script>
<?php
}
function get_file_content_callback() {
    if (isset($_POST['file_name'])) {
        $file_name = sanitize_text_field($_POST['file_name']);
        $file_path = ABSPATH . 'wp-content/themes/everyware-theme-lavender/nav-site-builder-json/' . $file_name;

        if (file_exists($file_path)) {
            echo file_get_contents($file_path);
        } else {
            echo "Error: File not found.";
        }
    }
    wp_die(); // this is required to terminate immediately and return a proper response
}

add_action('wp_ajax_get_file_content', 'get_file_content_callback');

?>